Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type Counter ¶
type Counter struct {
// contains filtered or unexported fields
}
Counter count how many jobs have started and finished.
type Elapsed ¶
type Elapsed struct {
Counter
// contains filtered or unexported fields
}
Elapsed is a job middleware that extends the simple counter and calculates the total time, average time and the last time spent doing the job.
Click to show internal directories.
Click to hide internal directories.