Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var ErrNotRetryable = errors.New("not retryable")
Functions ¶
func WithErrorResult ¶
WithErrorResult is a job wrapper that allows to receive the error result of the job.
For example it can be used to build logging or monitoring pipelines on the job.
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.